#!/usr/bin/env bash

#
# Copyright (c) 2009 Peter Ljung <ljung.peter@gmail.com>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

# TODO: Test commit support
# TODO: Factor our CVS folder handling. Two identical sections!

usage()
{
cat << EOF
usage: $0 options

This script takes the output (via STDIN) defined by scvs.finder.sh and executes an cvs command on each of the files/folders entered. Note that ROOT folder is not included.
It is assumed that cvs has the following structure;
    cvs [ cvs_options ] cvs_command [ command_options ] [ command_args ]
"cvs, cvs_options, cvs_command and command_options" are entered via -c options to the script. command_args are identified through STDIN and will only support commands which takes files/folders as input.
Note that scvs-inverter.sh must be in path.

OPTIONS:
   -h   show help
   -c   cvs command (add, remove and commit) including command options
   -g   call cvs command grouped together for each folder 
   -v   verbose/debug output
   -s   simulate command
EOF
}

while getopts "hc:gvs" OPTION; do
  case $OPTION in
    h )  usage;exit 1;;
    c )  COMMAND=$OPTARG;;
    g )  GROUP=1;;
    v )  VERBOSE=1;;
    s )  SIMULATE=1;;
    ? )  usage;exit 1;;
  esac
done

# Store STDIN input in temp file
TMPFILE=`mktemp -t scvs1.XXXXXXXXXX` || exit 1
if [[ "$COMMAND" =~ .*remove.* ]]; then
  cat <&0 | scvs-inverter.sh > $TMPFILE 	# Invert input before stored (requires scvs-inverter.sh located in PATH)
else
  cat <&0 > $TMPFILE
fi

# Helper: Execute command
# Executor executes command (including paramenters) sent to e
# Parameters including space are escaped with ""
function e {
	CMD=""
	for p in "$@";
  do
	  p="${p/# /}"									# Strip beginning/trailing spaces
	  p="${p/% /}"
	  if [[ $p =~ " " ]]; then
		  CMD="$CMD \"$p\"";
		else
      CMD="$CMD $p";
		fi
  done
  if [[ $VERBOSE ]]; then
  	echo "eval: $CMD"
	fi
  eval $CMD
}

if [[ "$COMMAND" =~ .*add.* ]] || [[ "$COMMAND" =~ .*commit.* ]]; then
	
	COMMAND="${COMMAND/# /}"								# Strip beginning/trailing space
	COMMAND="${COMMAND/% /}"

  # cvs commit is recursive in itself therefore only files/folders directly under R; is allowed
  # It is also assumed that D; are listed last i.e. first R;(1) then F;(*) then D;(*)

  while read LINE; do
	
    # Handle CVS folders
  	if [[ "$LINE" =~ ^R\;* ]] || [[ "$LINE" =~ ^D\;* ]]; then
	  	if [[ "$LINE" =~ /CVS$ ]]; then
				CVS="true"
				if [[ $VERBOSE ]]; then
		  		echo "cvs_folder: ${LINE/*;/}"
				fi
				continue
	  	else
	    	unset CVS
	  	fi
		fi
    if [[ "$LINE" =~ ^F\;* ]] && [[ $CVS ]]; then
    	if [[ $VERBOSE ]]; then
	    	echo "skip: ${LINE/*;/}"
	  	fi
	  	continue
		fi
	
    if [[ "$LINE" =~ ^R\;* ]]; then
      ROOT=${LINE/R;/}
      e cd "${ROOT}"
    elif [[ "$LINE" =~ ^D\;* ]]; then
      FOLDER=${LINE/D;/}
      if [[ $GROUP ]] && [[ $FILES ]]; then
        e cvs $COMMAND ${FILES}
      fi
      e cd "${FOLDER}/.."
      e cvs $COMMAND "${FOLDER//*\//}"
      e cd "${FOLDER}"
      FILES=""
    elif [[ "$LINE" =~ ^F\;* ]]; then
      FILE=${LINE/F;/}
      if [[ $GROUP ]]; then
        if [[ ${#FILES} > 0 ]]; then
          FILES="${FILES} "
        fi
        FILES="${FILES}\"${FILE}\""
      else
        e cvs $COMMAND "${FILE}"
      fi
    else
      echo "ERROR: Line \"$LINE\" is not processed!"
    fi

  done < $TMPFILE

  # Add LAST files
  if [[ $GROUP ]] && [[ $FILES ]]; then
    e cvs $COMMAND $FILES
  fi

elif [[ "$COMMAND" =~ .*remove.* ]]; then

  # Note: Files are removed before its folder. Finder order is also assumed to be inverted
  while read LINE; do

    # Handle CVS folders
    if [[ "$LINE" =~ ^R\;* ]] || [[ "$LINE" =~ ^D\;* ]]; then
	  	if [[ "$LINE" =~ /CVS$ ]]; then
				CVS="true"
				if [[ $VERBOSE ]]; then
		  		echo "cvs_folder: ${LINE/*;/}"
				fi
				continue
	  	else
	    	unset CVS
	  	fi
		fi
    if [[ "$LINE" =~ ^F\;* ]] && [[ $CVS ]]; then
      if [[ $VERBOSE ]]; then
	    	echo "skip: ${LINE/*;/}"
	  	fi
	  	continue
		fi

	  if [[ "$LINE" =~ ^R\;* ]]; then
	    ROOT=${LINE/R;/}
	    e cd "$ROOT"
	  elif [[ "$LINE" =~ ^D\;* ]]; then
	    if [[ ${FOLDER} ]] || [[ ${ROOT} ]]; then
	      # Remove files if -g opions
	      if [[ $GROUP ]] && [[ $FILES ]]; then
	        e rm $FILES
	        e cvs $COMMAND $FILES
		  	fi
		  	# Remove folder but not ROOT folder
	      if [[ ${FOLDER} ]]; then	      
	        e cd "$FOLDER/.."
	        e rm "${FOLDER//*\//}"
	        e cvs $COMMAND "${FOLDER//*\//}"
	      fi
	      FILES=""
	    fi
	    FOLDER=${LINE/D;/}
	    e cd "$FOLDER"
	  elif [[ "$LINE" =~ ^F\;* ]]; then
	    FILE=${LINE/F;/}
	    if [[ ${GROUP} ]]; then
	      if [[ ${#FILES} > 0 ]]; then
	        FILES="$FILES "
	      fi
	      FILES="${FILES}\"${FILE}\""
	    else
	      e rm "$FILE"
	      e cvs $COMMAND "$FILE"
	    fi
	  else
	    echo "ERROR: Line \"$LINE\" is not processed!"
	  fi

  done < $TMPFILE

  # Remove LAST files if -g opions
  if [[ $GROUP ]] && [[ $FILES ]]; then
    e rm $FILES
    e cvs $COMMAND $FILES
  fi
  # Remove LAST folder but not ROOT folder
  if [[ ${FOLDER} ]]; then	      
    e cd "$FOLDER/.."
    e rm "${FOLDER//*\//}"
    e cvs $COMMAND "${FOLDER//*\//}"
  fi

fi

# Clean up
rm -f $TMPFILE

