#!/usr/bin/env bash

#
# Copyright (c) 2009 Peter Ljung <ljung.peter@gmail.com>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

usage()
{
cat << EOF
usage: $0 options folder

This script locates files for SwingCVS application.

OPTIONS:
   -h   show help
   -p   root path used by finder
   -f   starting folder for finder (same as 1 argument after options)
   -i   include filter e.g. "*.c,*.h"
   -e   exclude filter e.g. "*.ini,*~" (precedence over include)
   -v   verbose/debug output
EOF
}

while getopts "hp:f:i:e:v" OPTION; do
  case $OPTION in
    h )  usage;exit 1;;
    p )  ROOT=$OPTARG;;
    f )  FOLDER=$OPTARG;;
    i )  INCLUDE=$OPTARG;;
    e )  EXCLUDE=$OPTARG;;
    v )  VERBOSE=1;;
    ? )  usage;exit 1;;
  esac
done

# Use pwd as root if not specified explicitely
if [ -z "${ROOT}" ]; then
  ROOT="$PWD"
fi
ROOT="${ROOT/# /}"								# Strip beginning/trailing space
ROOT="${ROOT/% /}"
ROOT="${ROOT/%\//}"               # remove trailing / (% means comparison from end)

# If '-f' has not been entered use first argument
if [ -z "${FOLDER}" ]; then
	if [ "${!OPTIND}" ]; then
  	FOLDER="${!OPTIND}"
	fi
fi
FOLDER="${FOLDER/# /}"				# Strip beginning/trailing space
FOLDER="${FOLDER/% /}"
FOLDER="${FOLDER/%\//}"       # remove trailing / (% means comparison from end)

# Reformat "loose" regexp format to common UNIX egrep regexp format
if [ "${INCLUDE}" ]; then
	INCLUDE="${INCLUDE/# /}"				# Strip beginning/trailing space
	INCLUDE="${INCLUDE/% /}"	
  INCLUDE="${INCLUDE//,/|}"       # replace , with |
  INCLUDE="${INCLUDE//\./\\.}"    # replace . with \.
  INCLUDE="${INCLUDE//\*/.*}"     # replace * with .*
fi
if [ "${EXCLUDE}" ]; then
	EXCLUDE="${EXCLUDE/# /}"				# Strip beginning/trailing space
	EXCLUDE="${EXCLUDE/% /}"	
  EXCLUDE="${EXCLUDE//,/|}"
  EXCLUDE="${EXCLUDE//\./\\.}"
  EXCLUDE="${EXCLUDE//\*/.*}"
fi

if [ "${FOLDER}" ]; then

  FOLDER="$ROOT/$FOLDER"
  # Output root folder while removing /folder/.. and /.
  echo "R;$FOLDER" | sed 's/\/[^\/]*\/\.\.//' | sed 's/\/\.//'

  CMD="ls -lR \"$FOLDER\""
  CMD=$CMD" | grep -v ^[[:space:]]\*$"                 # Remove blank lines
  CMD=$CMD" | grep -v ^total"                          # Remove total lines
  CMD=$CMD" | grep -v ^d"                              # Remove directory files
  CMD=$CMD" | sed '/:$/s/^/D;/g'"                      # Add D; to folder lines
  CMD=$CMD" | sed 's/:$//g'"			       	       			 # Remove trailing :
  CMD=$CMD" | sed 's/^.*[0-9] /F;/g'"  								 # Replace beginning of file lines with F;
  if [ "${EXCLUDE}" ]; then
    CMD=$CMD" | egrep -v \"^F;.*$EXCLUDE\""	           # Exclude filter (only files can be excluded)
  fi
  if [ "${INCLUDE}" ]; then
    CMD=$CMD" | egrep \"^R;|^D;|^F;.*$INCLUDE\""       # Include filter (root and directories are kept)
  fi
  CMD=$CMD" | sed 's/\/[^\/]*\/\.\.//'"                # Remove /folder/..
  CMD=$CMD" | sed 's/\/\.//'"                          # Remove /.

  if [ "${VERBOSE}" ]; then
    echo "ROOT=$ROOT"
    echo "FOLDER=$FOLDER"
    echo "INCLUDE=$INCLUDE"
    echo "EXCLUDE=$EXCLUDE"
    echo "Execute: $CMD"
    echo ""
  fi

  eval "$CMD"

else

  echo "No file/folder was entered. File name input from STDIN not supported yet. Exit"
  exit 1

fi

